using System;
using System.Data;
using System.Windows.Forms;
using System.Drawing;

#if NETCFDESIGNTIME
using System.ComponentModel;
using System.CodeDom;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;

[assembly: System.CF.Design.RuntimeAssemblyAttribute("RoundGauge, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null")]
#endif
namespace Gauges
{
	public delegate void HighWarnHandler(object sender, bool EnteredZone);
	public delegate void LowWarnHandler(object sender, bool EnteredZone);
	public delegate void MaxValueHandler(object sender);
	public delegate void MinValueHandler(object sender);

	#if NETCFDESIGNTIME
	// this code is required to get the designer to wrap the control initialization code with a
	// BeginUpdate and a EndUpdate call.
    internal class RoundGaugeDomSerializer : CodeDomSerializer {
        public override object Deserialize(IDesignerSerializationManager manager, object codeObject) {
                CodeDomSerializer baseClassSerializer = (CodeDomSerializer)manager.
                GetSerializer(typeof(RoundGauge).BaseType, typeof(CodeDomSerializer));

            return baseClassSerializer.Deserialize(manager, codeObject);
        }
 
        public override object Serialize(IDesignerSerializationManager manager, object value) {
            /* Associate the component with the serializer in the same manner as with
                Deserialize */
            CodeDomSerializer baseClassSerializer = (CodeDomSerializer)manager.
                GetSerializer(typeof(RoundGauge).BaseType, typeof(CodeDomSerializer));
 
            object codeObject = baseClassSerializer.Serialize(manager, value);

			CodeStatementCollection newCode = new CodeStatementCollection();
 
            if (codeObject is CodeStatementCollection) 
			{
				CodeMethodInvokeExpression invokeExpr;
				
                CodeStatementCollection statements = (CodeStatementCollection)codeObject;
 
                // The code statement collection is valid, so add our Begin/EndUpdate calls.
				CodeThisReferenceExpression thisRef = new CodeThisReferenceExpression();
				CodeFieldReferenceExpression gaugeRef = new CodeFieldReferenceExpression(thisRef, manager.GetName(value));

				// move the comments and the "new" call
				for(int i = 0 ; i < 4 ; i++)
				{
					newCode.Add(statements[0]);
					statements.RemoveAt(0);
				}
				
				// add BeginInvoke
				invokeExpr = new CodeMethodInvokeExpression(gaugeRef, "BeginUpdate");
				newCode.Add(invokeExpr);

				// add the designer-generated code
				newCode.AddRange(statements);

				// add EndUpdate
				invokeExpr = new CodeMethodInvokeExpression(gaugeRef, "EndUpdate");
                newCode.Add(invokeExpr);
            }
            return newCode;
        }
    }
	#endif

	#if NETCFDESIGNTIME
	[DesignerSerializer(typeof(RoundGaugeDomSerializer), typeof(CodeDomSerializer))]
	#endif
	public class RoundGauge : System.Windows.Forms.Control
	{
		public event HighWarnHandler	HighWarn;
		public event LowWarnHandler		LowWarn;
		public event MaxValueHandler	GaugeMax;
		public event MinValueHandler	GaugeMin;
	
		private Color		bezelColor			= Color.SlateGray;
		private Color		lineColor			= Color.Black;
		private Color		highwarnColor		= Color.Red;
		private Color		lowwarnColor		= Color.Red;
		private Bitmap		bufferBitmap;
		private Font		valueFont;
		private RectangleF	valueRect;
		private RectangleF	valueBorder;
		private int			arcBottom			= 0;
		private int			valueDigits			= -1;
		private int			valueFontSize		= 10;
		private int			bezelWidth			= 5;
		private int			valueArc;
		private float		startAngle;
		private float		valueAngle;
		private int			minValue			= 0;
		private int			highwarnValue		= 80;
		private int			lowwarnValue		= 0;
		private int			maxValue			= 100;
		private int			lineBezelSpacing	= 10;
		private int			arcWidth			= 3;
		private int			tickInterval		= 10;
		private int			tickLength			= 10;
		private bool		updating;
		private Rectangle	face;
		private bool		showTickLabels		= true;
		private int			tickLabelInterval	= 1;
		private float		currentValue		= 0F;
		private int			tickLabelPadding	= 10;
		private double		unitsPerDegree		= 0F;
		private int			diameter			= 100;
		private Needle		needle;
		private short		needleWidth;
		private string		name;
		private Lamps		lamps;
		private int			lampdiameter;
		private bool		inLowRange	= false;
		private bool		inHighRange	= false;

		public RoundGauge()
		{			
			// set defaults
			updating = true;

			BackColor = Color.Cornsilk;
			ValueArc = 300;
			currentValue = 0;

			valueFont = new Font(FontFamily.GenericMonospace, valueFontSize, FontStyle.Regular);
			face = new Rectangle(bezelWidth, bezelWidth, diameter - (2 * bezelWidth), diameter - (2 * bezelWidth));
			bufferBitmap = new Bitmap(diameter, diameter);	
			needle = new Needle(this.Width, valueAngle, (Width / 2 - bezelWidth - lineBezelSpacing));
			needle.Width = needleWidth;

			lamps = new Lamps();

			UpdateNeedleLength();
			unitsPerDegree = (double)ValueArc /  (double)(maxValue - minValue);

			updating = false;
			this.Refresh();
		}

		#region ----- Hidden Properties -----
		#if DESIGN
		[Browsable(false),
		EditorBrowsable(EditorBrowsableState.Never),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override int Width { get{return base.Width;} set{base.Width = value;} }

		[Browsable(false),
		EditorBrowsable(EditorBrowsableState.Never),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override int Height { get{return base.Height;} set{base.Height = value;} }

		[Browsable(false),
		EditorBrowsable(EditorBrowsableState.Never),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string Text { get{return base.Text;} set{base.Text = value;} }
		#endif
		#endregion

		#region Lamps Property
		#if NETCFDESIGNTIME
		[
		Category("Appearance"),
		Description("A collection of indicator lamps."),
		DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)
		]
		#endif
		public Lamps Lamps
		{
			get
			{
				return lamps;
			}
		}

		#region Needle Properties
		#endregion
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Needle"),
		System.ComponentModel.DefaultValue(5),
		System.ComponentModel.Description("Specifies the width of the gauge needle base.")
		]
		#endif		
		public short NeedleWidth
		{
			get
			{
				return needleWidth;
			}
			set
			{
				needleWidth = value;
				needle.Width = needleWidth;

				if(!updating)
					this.Refresh();
			}
		}

		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Needle"),
		System.ComponentModel.Description("Specifies the Color of the gauge needle.")
		]
		#endif
		public Color NeedleColor
		{
			get
			{
				return needle.NeedleColor;
			}
			set
			{
				needle.NeedleColor = value;
			}
		}
		#endregion

		#region Drawing/Painting Methods
		public override void Refresh()
		{
			Graphics g = Graphics.FromImage(bufferBitmap);
			if(!(this.Parent == null))
				g.Clear(this.Parent.BackColor);

//			DrawBezel(g);
//			DrawLines(g);

			DetermineValueRect(ref valueRect, ref valueBorder);
			
			this.Invalidate();
			
			updating = false;
			base.Refresh();
		}

		public void BeginUpdate()
		{
			updating = true;
		}

		public void EndUpdate()
		{
			updating = false;
			this.Refresh();
		}

		private void DrawBezel(Graphics g)
		{
			SolidBrush bezbrush = new SolidBrush(bezelColor);
			SolidBrush brush = new SolidBrush(this.BackColor);
			
			g.FillEllipse(bezbrush, 0, 0, diameter, diameter);
			g.FillEllipse(brush, face);

			brush.Dispose();
		}

		private void UpdateNeedleLength()
		{
			needle.Length = (this.Width / 2) - lineBezelSpacing - tickLength;
		}

		private Point[] CreateArc(float StartAngle, float SweepAngle, int PointsInArc, int Radius, int xOffset, int yOffset, int LineWidth)
		{
			if(PointsInArc > 360)
				PointsInArc = 360;

			Point[] points = new Point[PointsInArc * 2];
			int xo;
			int yo;
			int xi;
			int yi;
			float degs;
			double rads;
			
			for(int p = 0 ; p < PointsInArc ; p++)
			{
				degs = StartAngle + ((SweepAngle / PointsInArc) * p);
				
				rads = (degs * (Math.PI / 180));

				xo = (int)(Radius * Math.Sin(rads));
				yo = (int)(Radius * Math.Cos(rads));
				xi = (int)((Radius - LineWidth) * Math.Sin(rads));
				yi = (int)((Radius - LineWidth) * Math.Cos(rads));

				xo += (Radius + xOffset);
				yo = Radius - yo + yOffset;
				xi += (Radius + xOffset);
				yi = Radius - yi + yOffset;

				points[p] = new Point(xo, yo);
				points[(PointsInArc * 2) - (p + 1)] = new Point(xi, yi);
			}

			return points;
		}

		private void DetermineValueRect(ref RectangleF rect, ref RectangleF border)
		{
			int w = 0;
			float x = 0;
			float y = 0;
			SizeF size;
			float offset;
			Graphics g = Graphics.FromImage(bufferBitmap);

			// we pad with 1/2 digit space on left and right
			w = (valueDigits < 0) ? currentValue.ToString().Length + 1 : valueDigits + 1;
			
			offset = g.MeasureString("0", valueFont).Width;
			size = g.MeasureString(new string('0', w), valueFont);

			// center horizontally
			x = (this.Width - size.Width) / 2;

			// vertically at arc bottom
			y = arcBottom - size.Height;

			border = new RectangleF(x, y, size.Width, size.Height);
			rect = new RectangleF(x + (offset * 0.5F), y, size.Width - offset, size.Height);
		}

		private void DrawValue(Graphics g)
		{
			// only do something if we are showing digits
			if(valueDigits == 0) return;

			string val = currentValue.ToString();
			
			if(valueDigits > 0)
			{
				val = val.PadLeft(valueDigits, '0');
			}			

			g.DrawRectangle(new Pen(bezelColor), Rectangle.Ceiling(valueBorder));
			g.DrawString(val, valueFont, new SolidBrush(lineColor), valueRect);
		}
		
		private void DrawLines(Graphics g)
		{
			Point[] points;
			float arcstart = 0F;
			float arcsweep = 0F;
			int xo = 0;
			int yo = 0;
			int xi = 0;
			int yi = 0;
			int xl = 0;
			int yl = 0;
			int tickvalue = 0;
			int tickcount = 0;
			bool drawlabel = true;
			SizeF labelSize;
			double rads;
			int r = (face.Width / 2) - lineBezelSpacing;
			int offset = bezelWidth + lineBezelSpacing;
			Pen pen;
			Brush brush;

			float arcRange = (float)maxValue - (float)minValue;
			
			arcstart = startAngle;
			arcBottom = 0;

			// draw redline arc
			if(lowwarnValue > minValue)
			{
				pen = new Pen(lowwarnColor);
				brush = new SolidBrush(lowwarnColor);

				arcsweep = Convert.ToInt32((float)valueArc * (((float)lowwarnValue - (float)minValue) / arcRange));

				points = CreateArc(arcstart, (int)arcsweep, (int)arcsweep, r, offset, offset, arcWidth);				

				// get lowest point of arc (for value alignment later)
				arcBottom = (points[0].Y > arcBottom) ? points[0].Y : arcBottom;
				
				g.FillPolygon(brush, points);
				g.DrawPolygon(pen, points);

				arcstart = startAngle + (int)arcsweep;
			}

			// draw main arc
			pen = new Pen(lineColor);
			brush = new SolidBrush(lineColor);

			arcsweep = Convert.ToInt32((float)valueArc * ((float)(highwarnValue - lowwarnValue) / arcRange));

			points = CreateArc(arcstart, (int)arcsweep, (int)arcsweep, r, offset, offset, arcWidth);

			// get lowest point of arc (for value alignment later)
			arcBottom = (points[0].Y > arcBottom) ? points[0].Y : arcBottom;

			g.FillPolygon(brush, points);
			g.DrawPolygon(pen, points);

			arcstart += (int)arcsweep;

			// draw redline arc
			if(highwarnValue < maxValue)
			{
				pen = new Pen(highwarnColor);
				brush = new SolidBrush(highwarnColor);

				arcsweep = Convert.ToInt32((float)valueArc * ((float)(maxValue - highwarnValue) / arcRange));

				points = CreateArc(arcstart, (int)arcsweep, (int)arcsweep, r, offset, offset, arcWidth);

				g.FillPolygon(brush, points);
				g.DrawPolygon(pen, points);
			}

			// draw tics and labels
			pen = new Pen(lineColor);
			brush = new SolidBrush(lineColor);

			float tickIntervalDegs = ((float)valueArc / ((float)maxValue - (float)minValue)) * (float)tickInterval;
			
			if(tickIntervalDegs < 1)
			{
				throw new Exception("Cannot have a TickInterval < 1 degree");
			}

			tickvalue = minValue;
			for(int t = (int)startAngle ; t <= (startAngle + valueArc) ; t += (int)tickIntervalDegs)
			{
				labelSize = g.MeasureString(tickvalue.ToString(), this.Font);
				rads = (t * (Math.PI / 180));

				drawlabel = (showTickLabels && ((tickcount % tickLabelInterval) == 0));

				xo = (int)(r * Math.Sin(rads));
				yo = (int)(r * Math.Cos(rads));
				xo += (r + offset);
				yo = r - yo + offset;
				
				if(drawlabel)
				{
					xi = (int)((r - (tickLength * 1.5)) * Math.Sin(rads));
					yi = (int)((r - (tickLength * 1.5)) * Math.Cos(rads));
					xi += (r + offset);
					yi = r - yi + offset;
				}
				else
				{
					xi = (int)((r - tickLength) * Math.Sin(rads));
					yi = (int)((r - tickLength) * Math.Cos(rads));
					xi += (r + offset);
					yi = r - yi + offset;
				}

				g.DrawLine(pen, xo, yo, xi, yi);
				
				if(drawlabel)
				{
					xl = (int)((r - ((tickLength * 2) + tickLabelPadding)) * Math.Sin(rads));
					yl = (int)((r - ((tickLength * 2) + tickLabelPadding)) * Math.Cos(rads));
					xl += (r + offset - (int)(labelSize.Width / 2));
					yl = r - yl + offset - (int)(labelSize.Height / 2);

					g.DrawString(tickvalue.ToString(), this.Font, brush, xl, yl);
				}

				tickcount++;
				tickvalue += tickInterval;
			}

			// clean up
			pen.Dispose();
			brush.Dispose();	
		}

		protected override void OnPaintBackground(PaintEventArgs e)
		{
		#if NETCFDESIGNTIME
			// prevent designer artifacts
			e.Graphics.Clear(this.Parent.BackColor);
			needle.Draw(e.Graphics);
		#endif
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			// double buffer the output
			Bitmap b = new Bitmap(bufferBitmap);
			Graphics g = Graphics.FromImage(b);
			DrawValue(g);
			needle.Angle = valueAngle;
			needle.Draw(g);
			
			foreach(Lamp l in lamps)
			{
				if(l.Visible)
				{
					l.Draw(g);
				}
			}

			// draw the gauge
			e.Graphics.DrawImage(b, 0, 0);
			b.Dispose();
			g.Dispose();
		}
		#endregion

		#region ControlName
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Design"),
		System.ComponentModel.Description("Specifies the name of the Control.")
		]
		#endif
		public string ControlName
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}
		#endregion

		#region Diameter
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance"),
		System.ComponentModel.Description("Specifies the width and height of the gauge.")
		]
		#endif
		public int Diameter
		{
			get
			{
				return diameter;
			}
			set
			{
				diameter = value;
				Width = diameter;
				Height = diameter;
				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region Value
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Values"),
		System.ComponentModel.Description("Specifies the value to which the gauge points.")
		]
		#endif
		public float Value
		{
			get
			{
				return currentValue;
			}
			set
			{
				currentValue = value;
				valueAngle = startAngle + (int)((currentValue - minValue) * unitsPerDegree);

				if(valueDigits == -1)
				{
					DetermineValueRect(ref valueRect, ref valueBorder);
				}

				#region event handling
				if((GaugeMax != null) && (currentValue >= MaxValue))
				{
					GaugeMax(this);
				}

				if((GaugeMin != null) && (currentValue <= MinValue))
				{
					GaugeMin(this);
				}

				if((LowWarn != null) && (LowWarnValue > MinValue))
				{
					if(currentValue < LowWarnValue)
					{
						if(! inLowRange)
						{
							// just passed into low
							LowWarn(this, true);
							inLowRange = true;
						}
					}
					else
					{
						if(inLowRange)
						{
							// just passed out of low
							LowWarn(this, false);
							inLowRange = false;
						}
					}
				}

				if((HighWarn != null) && (HighWarnValue > MinValue))
				{
					if(currentValue > HighWarnValue)
					{
						if(! inHighRange)
						{
							// just passed into High
							HighWarn(this, true);
							inHighRange = true;
						}
					}
					else
					{
						if(inHighRange)
						{
							// just passed out of High
							HighWarn(this, false);
							inHighRange = false;
						}
					}
				}
				#endregion Event Handling

				base.Invalidate();
			}
		}
		#endregion
		#region ValueArc
		#if NETCFDESIGNTIME
				[
				System.ComponentModel.Category("Values"),
				System.ComponentModel.Description("Arc in degrees over which value line and tick will be drawn.")
				]
		#endif
		public int ValueArc
		{
			get 
			{
				return valueArc;
			}
			set 
			{
				startAngle = 180 + ((360 - value) / 2);
				valueAngle = startAngle;
				valueArc = value;
				unitsPerDegree = (double)ValueArc /  (double)(maxValue - minValue);

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region TickLabelInterval
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Ticks"),
		System.ComponentModel.Description("Specifies the number of ticks between TickLabels.")
		]
		#endif
		public int TickLabelInterval
		{
			get
			{
				return tickLabelInterval;
			}
			set
			{
				tickLabelInterval = value;

				if(!updating)
					this.Refresh();
			}

		}
		#endregion
		#region ShowTickLabels
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Ticks"),
		System.ComponentModel.Description("Specifies whether or not TickLabels will be displayed.")
		]
		#endif
		public bool ShowTickLabels
		{
			get 
			{
				return showTickLabels;
			}
			set 
			{
				showTickLabels = value;

				if(!updating)
					this.Refresh();
			}

		}
		#endregion
		#region LineBezelSpacing
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance"),
		System.ComponentModel.Description("Specifies the gap between the ValueArc and the gauge bezel.")
		]
		#endif
		public int LineBezelSpacing
		{
			get 
			{
				return lineBezelSpacing;
			}
			set 
			{
				lineBezelSpacing = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region ArcWidth
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance"),
		System.ComponentModel.Description("Specifies the thickness of the ValueArc line.")
		]
		#endif
		public int ArcWidth
		{
			get 
			{
				return arcWidth;
			}
			set 
			{
				arcWidth = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region TickInterval
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Ticks"),
		System.ComponentModel.Description("Specifies the interval, based on Value, between individual Tick lines.")
		]
		#endif
		public int TickInterval
		{
			get 
			{
				return tickInterval;
			}
			set 
			{
				tickInterval = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region TickLabelPadding
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Ticks"),
		System.ComponentModel.Description("Specifies the gap between the Tick lines and the TickLabels.")
		]
		#endif
		public int TickLabelPadding
		{
			get 
			{
				return tickLabelPadding;
			}
			set 
			{
				tickLabelPadding = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region TickLength
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Ticks"),
		System.ComponentModel.Description("Specifies the length of standard Tick lines.  Tick lines with a label are twice this value.")
		]
		#endif
		public int TickLength
		{
			get 
			{
				return tickLength;
			}
			set 
			{
				tickLength = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region MinValue
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Values"),
		System.ComponentModel.Description("Specifies the minimum value displayed on the gauge.")
		]
		#endif
		public int MinValue
		{
			get 
			{
				return minValue;
			}
			set 
			{
				minValue = value;

				if(lowwarnValue < minValue)
				{
					lowwarnValue = minValue;
				}
				
				unitsPerDegree = (double)ValueArc /  (double)(maxValue - minValue);

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region HighWarnValue
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Values"),
		System.ComponentModel.Description("Specifies the lower limit of the HighWarn region.")
		]
		#endif
		public int HighWarnValue
		{
			get 
			{
				return highwarnValue;
			}
			set 
			{
				if(highwarnValue > maxValue)
				{
					highwarnValue = maxValue;
					#if DESIGN
					MessageBox.Show("HighWarnValue cannot exceed maxValue.");
					#endif
				}

				highwarnValue = value;

				if(!updating)
					this.Refresh();
			}
		}
		
		#endregion
		#region LowWarnValue
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Values"),
		System.ComponentModel.Description("Specifies the upper limit of the LowWarn region.")
		]
		#endif
		public int LowWarnValue
		{
			get
			{
				return lowwarnValue;
			}
			set 
			{
				if(lowwarnValue < minValue)
				{
					lowwarnValue = minValue;
				}

				lowwarnValue = value;

				if(!updating)
					this.Refresh();
			}

		}

		#endregion
		#region MaxValue
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Values"),
		System.ComponentModel.Description("Specifies the maximum value displayed on the gauge.")
		]
		#endif
		public int MaxValue
		{
			get 
			{
				return maxValue;
			}
			set 
			{
				maxValue = value;
				
				if(highwarnValue > maxValue)
				{
					highwarnValue = maxValue;
				}
				unitsPerDegree = (double)ValueArc /  (double)(maxValue - minValue);

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region ValueDigits
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Values"),
		System.ComponentModel.Description("Specifies the number of digits to display in the value label.  0 will hide the value label.  -1 sets it to autosize.")
		]
		#endif
		public int ValueDigits
		{
			get
			{
				return valueDigits;
			}
			set
			{
				valueDigits = value;
				DetermineValueRect(ref valueRect, ref valueBorder);

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region BezelWidth
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance"),
		System.ComponentModel.Description("Specifies the thickness of the gauge bezel.")
		]
		#endif
		public int BezelWidth
		{
			get 
			{
				return bezelWidth;
			}
			set 
			{
				bezelWidth = value;
				face = new Rectangle(bezelWidth, bezelWidth, this.Width - (2 * bezelWidth), this.Height - (2 * bezelWidth));

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region BezelColor
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance"),
		System.ComponentModel.Description("Specifies the Color of the gauge bezel.")
		]
		#endif
		public Color BezelColor
		{
			get 
			{
				return bezelColor;
			}
			set 
			{
				bezelColor = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region LineColor
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance"),
		System.ComponentModel.Description("Specifies the Color of Ticks and the ValueArc.")
		]
		#endif
		public Color LineColor
		{
			get 
			{
				return lineColor;
			}
			set 
			{
				lineColor = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region LowWarnColor
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance"),
		System.ComponentModel.Description("Specifies the Color of the ValueArc in the LowWarn region.")
		]
		#endif
		public Color LowWarnColor
		{
			get 
			{
				return lowwarnColor;
			}
			set 
			{
				lowwarnColor = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion
		#region HighWarnColor
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance"),
		System.ComponentModel.Description("Specifies the Color of the ValueArc in the HighWarn region.")
		]
		#endif
		public Color HighWarnColor
		{
			get 
			{
				return highwarnColor;
			}
			set 
			{
				highwarnColor = value;

				if(!updating)
					this.Refresh();
			}
		}
		#endregion

		protected override void OnResize(EventArgs e)
		{
			diameter = this.Height = this.Width;

			face = new Rectangle(bezelWidth, bezelWidth, diameter - (2 * bezelWidth), diameter - (2 * bezelWidth));
			bufferBitmap = new Bitmap(diameter, diameter);
			DetermineValueRect(ref valueRect, ref valueBorder);
			needle = new Needle(this.Width, valueAngle, (Width / 2 - bezelWidth - lineBezelSpacing));
			needle.Width = needleWidth;

			// update lamp data
			int lampx = 0, lampy = 0;
			lampdiameter = (int)((this.Width / 2)  - ((this.Width / 2) / 1.4142135623730950488016887242097));

			foreach(Lamp l in lamps)
			{
				switch(l.Position)
				{
					case LampPosition.UpperLeft:
						lampx = lampdiameter + 1;
						lampy = lampdiameter + 1;
						break;
					case LampPosition.UpperRight:
						lampx = this.Width - 1;
						lampy = lampdiameter + 1;
						break;
					case LampPosition.LowerLeft:
						lampx = lampdiameter + 1;
						lampy = this.Height - 1;
						break;
					case LampPosition.LowerRight:
						lampx = this.Width - 1;
						lampy = this.Height - 1;
						break;
				}
				l.UpperRight = new Point(lampx, lampy);
				l.Diameter = lampdiameter;
			}

			base.OnResize (e);

			if(!updating)
				this.Refresh();
		}

		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
	}

}
